@@ -1,3 +1,4 @@
+from __future__ import print_function
 # Licensed to the Apache Software Foundation (ASF) under one or more
 # contributor license agreements.  See the NOTICE file distributed with
 # this work for additional information regarding copyright ownership.
@@ -241,19 +242,21 @@ Enter a raw query, without processing the returned HTML contents.
 """
 import sys
 import socket
-import httplib
-import urlparse
 import codecs
-import urllib
 import datetime
 import logging
-from StringIO import StringIO
+import six
+import base64
 from xml.sax import make_parser
 from xml.sax.handler import ContentHandler
 from xml.sax.saxutils import escape, quoteattr
 from xml.dom.minidom import parseString
-
-__version__ = "0.9.6"
+from past.builtins import long, unicode, basestring, PY3
+from future.utils import iteritems
+from six import BytesIO as StringIO
+import six.moves.http_client as httplib
+import six.moves.urllib.parse as urlparse
+import six.moves.urllib.parse as urllib
 
 __all__ = ['SolrException', 'Solr', 'SolrConnection',
            'Response', 'SearchHandler']
@@ -419,10 +422,14 @@ class Solr:
 
         self.form_headers = {
             'Content-Type': 'application/x-www-form-urlencoded; charset=utf-8'}
-        
+        self.form_headers.update(post_headers)
+
+        # Patch: https://github.com/search5/solrpy/issues/51
         if http_user is not None and http_pass is not None:
             http_auth = http_user + ':' + http_pass
-            http_auth = 'Basic ' + http_auth.encode('base64').strip()
+            if six.PY3:
+                http_auth = http_auth.strip().encode('utf-8')
+            http_auth = b'Basic ' + base64.b64encode(http_auth)
             self.auth_headers = {'Authorization': http_auth}
         else:
             self.auth_headers = {}
@@ -509,6 +516,7 @@ class Solr:
         for doc in docs:
             self.__add(lst, doc)
         lst.append(u'</add>')
+        print(''.join(lst))
         return ''.join(lst)
 
     def commit(self, wait_flush=True, wait_searcher=True, _optimize=False):
@@ -555,6 +563,8 @@ class Solr:
 
         # Detect old-style error response (HTTP response code
         # of 200 with a non-zero status).
+        if PY3:
+            data = data.decode("utf-8")
         starts = data.startswith
         if starts('<result status="') and not starts('<result status="0"'):
             data = self.decoder(data)[0]
@@ -567,7 +577,7 @@ class Solr:
 
     def __add(self, lst, fields):
         lst.append(u'<doc>')
-        for field, value in fields.items():
+        for (field, value) in iteritems(fields):
             # Handle multi-valued fields if values
             # is passed in as a list/tuple
             if not isinstance(value, (list, tuple, set)):
@@ -589,9 +599,11 @@ class Solr:
                 elif isinstance(value, bool):
                     value = value and 'true' or 'false'
 
-                lst.append('<field name=%s>%s</field>' % (
-                    (quoteattr(field),
-                    escape(unicode(value)))))
+                elem = FieldElement()
+                elem['name'] = quoteattr(field)
+                elem['value'] = escape(unicode(value))
+
+                lst.append(unicode(elem))
         lst.append('</doc>')
 
     def _delete(self, id=None, ids=None, queries=None):
@@ -793,10 +805,13 @@ class SearchHandler(object):
 
         params['fl'] = fields
         params['version'] = self.conn.response_version
-        params['wt'] = 'standard'
+        params['wt'] = 'xml'
 
-        xml = self.raw(**params)
-        return parse_query_response(StringIO(xml),  params, self)
+        json = self.raw(**params)
+        if PY3 and type(json) == str:
+            json = json.encode("utf-8")
+        return parse_query_response("XML", StringIO(json),  params, self)
+        # return parse_query_response("JSON", StringIO(json), params, self)
 
     def raw(self, **params):
         """
@@ -807,7 +822,7 @@ class SearchHandler(object):
         """
         # Clean up optional parameters to match SOLR spec.
         query = []
-        for key, value in params.items():
+        for (key, value) in iteritems(params):
             key = key.replace(self.arg_separator, '.')
             if isinstance(value, (list, tuple)):
                 query.extend([(key, strify(v)) for v in value])
@@ -948,21 +963,24 @@ class Response(object):
 # ===================================================================
 # XML Parsing support
 # ===================================================================
-def parse_query_response(data, params, query):
+def parse_query_response(data_type, data, params, query):
     """
     Parse the XML results of a /select call.
     """
-    parser = make_parser()
-    handler = ResponseContentHandler()
-    parser.setContentHandler(handler)
-    parser.parse(data)
-    if handler.stack[0].children:
-        response = handler.stack[0].children[0].final
-        response._params = params
-        response._query = query
-        return response
-    else:
-        return None
+    if data_type == "XML":
+        parser = make_parser()
+        handler = ResponseContentHandler()
+        parser.setContentHandler(handler)
+        parser.parse(data)
+        if handler.stack[0].children:
+            response = handler.stack[0].children[0].final
+            response._params = params
+            response._query = query
+            return response
+        else:
+            return None
+    elif data_type == "JSON":
+        pass
 
 
 class ResponseContentHandler(ContentHandler):
@@ -1047,14 +1065,13 @@ class ResponseContentHandler(ContentHandler):
         elif name == 'result':
             node.final = Results([cnode.final for cnode in node.children])
 
-
         elif name in ('responseHeader',):
             node.final = dict([(cnode.name, cnode.final)
                         for cnode in node.children])
         else:
             raise SolrException("Unknown tag: %s" % name)
 
-        for attr, val in node.attrs.items():
+        for (attr, val) in iteritems(node.attrs):
             if attr != 'name':
                 setattr(node.final, attr, val)
 
@@ -1086,7 +1103,55 @@ class Node(object):
             self.name,
             "".join(self.chars).strip(),
             ' '.join(['%s="%s"' % (attr, val)
-                            for attr, val in self.attrs.items()]))
+                            for (attr, val) in iteritems(self.attrs)]))
+
+
+class FieldElement:
+    def __init__(self):
+        self.attrs = {}
+
+    def __setitem__(self, key, value):
+        self.attrs[key] = unicode(value)
+
+        if key == "value" and value.startswith("{'"):
+            tmp = eval(value)
+            allow_keys = ['add', 'set', 'inc']
+            if len(tmp) > 1:
+                raise NotFieldOption('Only one option is available.')
+
+            first_key = tmp.popitem()
+
+            if first_key[0] not in allow_keys:
+                raise NotFieldOption('This option is not allowed.')
+
+            self.attrs["update"] = '"{0}"'.format(first_key[0])
+            if type(first_key[1]) == list:
+                # self.attrs["multi_update"] = self.attrs["update"]
+                self.attrs["multi_value"] = first_key[1]
+            self.attrs["value"] = unicode(first_key[1])
+
+    def get_attr(self):
+        attrs = [u'{0}={1}'.format(attr_name, attr_value)
+                 for attr_name, attr_value in self.attrs.items()
+                 if attr_name not in ('value', 'multi_value')]
+        return ' '.join(attrs)
+
+    def __unicode__(self):
+        if 'multi_value' in self.attrs:
+            res_fields = []
+
+            for entry in self.attrs['multi_value']:
+                res_fields.append(u'<field{0}>{1}</field>'.format(u' {0}'.format(self.get_attr()), unicode(entry)))
+
+            return u'\n'.join(res_fields)
+        else:
+            return u'<field{0}>{1}</field>'.format(u' {0}'.format(self.get_attr()), self.attrs['value'])
+
+    def __str__(self):
+        return self.__unicode__()
+
+
+class NotFieldOption(Exception): pass
 
 
 # ===================================================================
@@ -1097,6 +1162,7 @@ def check_response_status(response):
         ex = SolrException(response.status, response.reason)
         try:
             ex.body = response.read()
+            print(ex.body)
         except:
             pass
         raise ex
@@ -1160,7 +1226,7 @@ def qs_from_items(query):
     qs = ''
     if query:
         sep = '?'
-        for k, v in query.items():
+        for (k, v) in iteritems(query):
             k = urllib.quote(k)
             if isinstance(v, basestring):
                 v = [v]
