@@ -23,7 +23,7 @@ class SolrPaginator:
     """
 
     def __init__(self, result, default_page_size=None):
-        self.params = result.header['params']
+        self.params = result._params
         self.result = result
         self.query = result._query
 
@@ -65,7 +65,7 @@ class SolrPaginator:
         # need to convert the keys to strings to pass them as parameters
         new_params = {}
         for k, v in self.params.items():
-            new_params[str(k)] = v.encode('utf-8')
+            new_params[str(k)] = v
 
         # get the new start index
         new_params['start'] = start
@@ -79,7 +79,7 @@ class SolrPaginator:
             raise 'PageNotAnInteger'
 
         if page_num not in self.page_range:
-            raise 'EmptyPage', 'That page does not exist.'
+            raise EmptyPage('That page does not exist.')
 
         # Page 1 starts at 0; take one off before calculating
         start = (page_num - 1) * self.page_size
@@ -130,3 +130,4 @@ class SolrPage:
     def previous_page_number(self):
         return self.number - 1
 
+class EmptyPage(Exception): pass
